@@ -5,6 +5,7 @@ import os
 import re
 import subprocess
 import shlex
+import StringIO
 import unittest
 
 from nss.error import NSPRError
@@ -15,8 +16,8 @@ import nss.nss as nss
 
 verbose = False
 db_name = 'sql:pki'
-db_passwd = 'db_passwd'
-pk12_passwd = 'pk12_passwd'
+db_passwd = 'DB_passwd'
+pk12_passwd = 'PK12_passwd'
 
 cert_nickname = 'test_user'
 pk12_filename = '%s.p12' % cert_nickname
@@ -115,12 +116,17 @@ def strip_key_from_pk12_listing(text):
         raise ValueError('Could not file Key section in pk12 listing')
     return text[match.start(0):]
 
+def strip_salt_from_pk12_listing(text):
+    return re.sub(r'\s+Salt:\s*\n.*', '', text)
+
 #-------------------------------------------------------------------------------
 
 def load_tests(loader, tests, pattern):
     suite = unittest.TestSuite()
     tests = loader.loadTestsFromNames(['test_pkcs12.TestPKCS12Decoder.test_read',
-                                       'test_pkcs12.TestPKCS12Decoder.test_import',
+                                       'test_pkcs12.TestPKCS12Decoder.test_import_filename',
+                                       'test_pkcs12.TestPKCS12Decoder.test_import_fileobj',
+                                       'test_pkcs12.TestPKCS12Decoder.test_import_filelike',
                                        'test_pkcs12.TestPKCS12Export.test_export',
                                        ])
     suite.addTests(tests)
@@ -172,8 +178,8 @@ class TestPKCS12Decoder(unittest.TestCase):
 
         self.assertEqual(cert_bag_count, 2)
 
-    def test_import(self):
-        if verbose: print "test_import"
+    def test_import_filename(self):
+        if verbose: print "test_import_filename"
         delete_cert_from_db(cert_nickname)
         self.assertEqual(get_cert_der_from_db(cert_nickname), None)
 
@@ -184,6 +190,39 @@ class TestPKCS12Decoder(unittest.TestCase):
         cert_der = get_cert_der_from_db(cert_nickname)
         self.assertEqual(cert_der, self.cert_der)
 
+    def test_import_fileobj(self):
+        if verbose: print "test_import_fileobj"
+        delete_cert_from_db(cert_nickname)
+        self.assertEqual(get_cert_der_from_db(cert_nickname), None)
+
+        slot = nss.get_internal_key_slot()
+
+        file_obj = open(pk12_filename)
+
+        pkcs12 = nss.PKCS12Decoder(file_obj, pk12_passwd, slot)
+        file_obj.close()
+        slot.authenticate()
+        pkcs12.database_import()
+        cert_der = get_cert_der_from_db(cert_nickname)
+        self.assertEqual(cert_der, self.cert_der)
+
+    def test_import_filelike(self):
+        if verbose: print "test_import_filelike"
+        delete_cert_from_db(cert_nickname)
+        self.assertEqual(get_cert_der_from_db(cert_nickname), None)
+
+        slot = nss.get_internal_key_slot()
+
+        with open(pk12_filename) as f:
+            data = f.read()
+        file_obj = StringIO.StringIO(data)
+
+        pkcs12 = nss.PKCS12Decoder(file_obj, pk12_passwd, slot)
+        slot.authenticate()
+        pkcs12.database_import()
+        cert_der = get_cert_der_from_db(cert_nickname)
+        self.assertEqual(cert_der, self.cert_der)
+
 #-------------------------------------------------------------------------------
 
 class TestPKCS12Export(unittest.TestCase):
@@ -206,9 +245,11 @@ class TestPKCS12Export(unittest.TestCase):
 
         pk12_listing = list_pk12(pk12_filename)
         pk12_listing = strip_key_from_pk12_listing(pk12_listing)
+        pk12_listing = strip_salt_from_pk12_listing(pk12_listing)
 
         exported_pk12_listing = list_pk12(exported_pk12_filename)
         exported_pk12_listing = strip_key_from_pk12_listing(exported_pk12_listing)
+        exported_pk12_listing = strip_salt_from_pk12_listing(exported_pk12_listing)
 
         self.assertEqual(pk12_listing, exported_pk12_listing)
 
